import asyncio
import json

from quart import Quart, make_response
from urllib.parse import urljoin

from .data_provider import (
    DataProvider,
    bovine_actor_for_actor_data,
)
from .server import BlueprintForActors, assets_blueprint_for_directory

import click


def bovine_actor_and_actor_object(domain: str):
    """Returns the BovineActor and the Actor object corresponding to the
    actor being served by one_actor"""
    dp = DataProvider.load()

    actor_id = urljoin(domain, "/actor")

    return bovine_actor_for_actor_data(actor_id, dp.one_actor)


@click.command()
@click.option("--only_generate_config", default=False, is_flag=True)
@click.option(
    "--assets",
    default=None,
    help="Directory to serve assets from. Assets will be served under /assets/",
)
@click.option("--port", default=2909, help="port to run on")
def one_actor(only_generate_config, assets, port):
    dp = DataProvider.generate_and_load(only_generate_config)
    app = Quart(__name__)

    queue = asyncio.Queue()

    async def generator():
        yield """<html><body><h1>Inbox content</h1>"""
        while True:
            data = await queue.get()
            if data:
                yield f"""<p><pre>{json.dumps(data, indent=2)}</pre></p>"""

    async def on_inbox(data):
        await queue.put(data)

    if assets:
        app.register_blueprint(assets_blueprint_for_directory(assets))

    app.register_blueprint(
        BlueprintForActors(actors=[dp.one_actor], on_inbox=on_inbox).blueprint
    )

    @app.get("/")
    async def inbox_display():
        response = await make_response(generator())
        response.timeout = None
        return response

    app.run(port=port, host="0.0.0.0", use_reloader=False)


if __name__ == "__main__":
    one_actor()



from contextlib import asynccontextmanager

from tortoise import Tortoise

from .models import TestRecord


class ResultStore:
    async def add_result(self, test_name: str, application_name: str, data: dict):
        """Adds a result to the database. The pairs (test_name, application_name)
        are assumed to be unique"""
        await TestRecord.update_or_create(
            test_name=test_name,
            application_name=application_name,
            defaults={"data": data},
        )

    async def results_for_test(self, test_name: str) -> list:
        """Retrieves the results for a given test_name"""
        result = await TestRecord.filter(test_name=test_name).all()

        return [{"application_name": x.application_name, **x.data} for x in result]


@asynccontextmanager
async def with_store(db_url="sqlite://test_results.sqlite") -> ResultStore:
    """Initializes the database and returns a ResultStore. Usage:

    ```python
    async with with_store() as store:
        await store.add_result(...)
        ...
        await store.results_for_test(...)
    ```
    """
    await Tortoise.init(
        config={
            "connections": {"default": db_url},
            "apps": {
                "models": {
                    "models": [
                        "fediverse_pasture.runner.models",
                    ],
                    "default_connection": "default",
                },
            },
        },
    )
    await Tortoise.generate_schemas()

    yield ResultStore()

    await Tortoise.close_connections()



import os

from .result_store import with_store


async def test_with_store():
    file = "test3.sqlite"

    async with with_store(db_url=f"sqlite://{file}") as store:
        await store.add_result("test", "app1", {})
        await store.add_result("test", "app1", {"a": "b"})
        await store.add_result("test", "app2", {})
        await store.add_result("other", "app1", {})

        result = await store.results_for_test("test")

        assert len(result) == 2

    os.unlink(file)
